@@ -5,6 +5,7 @@ from typing import Literal, Union, Optional, Sequence, Dict
 import math
 from pathlib import Path
 from collections import namedtuple
+from itertools import accumulate
 import importlib.resources as pkg_resources
 import xml.etree.ElementTree as ET
 
@@ -16,9 +17,9 @@ from .cmap import Cmap12, Cmap4
 from .glyph import SimpleGlyph, CompoundGlyph
 from .glyphcff import read_glyph_cff, CFF
 from .glyphglyf import read_glyph_glyf
-from .glyphinspect import ShowGlyphs
+from .findfont import find_font
 from .fonttypes import (AdvanceWidth, Layout, Header, Table,
-                        FontInfo, FontNames, Symbols, FontFeatures)
+                        FontInfo, FontNames, Symbols)
 from .svgpath import fmt
 
 
@@ -28,13 +29,16 @@ class Font:
         Args:
             name: File name of the font. Defaults to DejaVuSans.
     '''
-    def __init__(self, name: Optional[Union[str, Path]] = None):
+    def __init__(self, name: Optional[Union[str, Path]] = None,
+                 searchpaths: Optional[Sequence[str | Path]] = None):
         self.fname = None
         if name:
             if Path(name).exists():
                 self.fname = Path(name)
             else:
-                raise FileNotFoundError(f'Font {name} not found.')
+                self.fname = find_font(name, searchpaths)
+                if self.fname is None:
+                    raise FileNotFoundError(f'Font {name} not found.')
         else:
             with pkg_resources.path('ziafont.fonts', 'DejaVuSans.ttf') as p:
                 self.fname = p
@@ -43,7 +47,7 @@ class Font:
             self.fontfile = FontReader(f.read())
 
         self.tables: dict[str, Table] = {}
-        self.features = FontFeatures()
+        self.features: dict[str, bool] = {}
         self.info = self._loadfont()  # Load in all the font metadata
         self._glyphs: Dict[int, Union[SimpleGlyph, CompoundGlyph]] = {}
         self._glyphids: Dict[str, int] = {}
@@ -60,10 +64,12 @@ class Font:
         self.gpos = None
         if 'GPOS' in self.tables:
             self.gpos = gpos.Gpos(self.tables['GPOS'].offset, self.fontfile)
+            self.features.update(self.gpos.init_user_features())
 
         self.gsub = None
         if 'GSUB' in self.tables:
             self.gsub = gsub.Gsub(self.tables['GSUB'].offset, self.fontfile)
+            self.features.update(self.gsub.init_user_features())
 
         self.cffdata: Optional[CFF] = None
         if 'CFF ' in self.tables:
@@ -299,8 +305,9 @@ class Font:
 
     def language(self, script, language):
         ''' Set script/language to use '''
-        if script == 'DFLT' and 'DFLT' not in self.scripts():
-            script = 'latn'
+        scripts = self.scripts()
+        if script == 'DFLT' and 'DFLT' not in scripts:
+            script = 'latn' if 'latn' in scripts else scripts[0]
 
         if script not in self.scripts():
             raise ValueError(f'Script {script} not defined in font')
@@ -308,12 +315,15 @@ class Font:
         if language not in self.languages(script):
             raise ValueError(f'Language {language} not defined in font')
 
+        self.features = {}
         if self.gpos:
             self.gpos.language.script = script
             self.gpos.language.language = language
+            self.features.update(self.gpos.init_user_features())
         if self.gsub:
             self.gsub.language.script = script
             self.gsub.language.language = language
+            self.features.update(self.gsub.init_user_features())
 
     def glyphindex(self, char: str) -> int:
         ''' Get index of character glyph '''
@@ -343,18 +353,20 @@ class Font:
             self._glyphs[glyphid] = glyph
         return glyph
 
-    def advance(self, glyph1: int, glyph2: Optional[int] = None):
-        ''' Get advance width in font units, including kerning adjustment
-            if glyph2 is defined
-        '''
+    def advance(self, glyph: int, glyph2: int|None) -> int:
+        ''' Get advance width in font units '''
         try:
-            adv = self.advwidths[glyph1].width
+            adv = self.advwidths[glyph].width
         except IndexError:
             adv = self.info.layout.advwidthmax.width
 
-        if self.features.kern and glyph2 and self.gpos:
+        if self.features.get('kern', True) and glyph2 and self.gpos:
             # Only getting x-advance for first glyph.
-            adv += self.gpos.kern(glyph1, glyph2)[0].get('xadvance', 0)
+            pos = self.gpos.position(
+                [self.glyph_fromid(glyph), self.glyph_fromid(glyph2)],
+                {'kern': True})
+            adv = pos[1][0]
+
         return adv
 
     def getsize(self, s) -> tuple[float, float]:
@@ -389,9 +401,6 @@ class Font:
                    color=color, rotation=rotation, rotation_mode=rotation_mode)
         return txt
 
-    def showallglyphs(self, size: float = 36, pxwidth: int = 800):
-        return ShowGlyphs(self)
-
 
 class Text:
     ''' Draw text as SVG paths
@@ -570,51 +579,50 @@ class Text:
         gidlines = self.str_to_gids()
         yvals = [i*lineheight for i in range(len(gidlines))]  # valign == 'base'
 
-        # Generate symbols and calculate x positions using left alignment
         symbols: list[ET.Element] = []  # <symbol> elements
         linewidths: list[float] = []
-        xadvance = 0.
-        allglyphs = []  # (glyph, x, y) where x is left aligned, y is delta from baseline
+        allglyphs: list[list[tuple[SimpleGlyph, int, int]]] = []  # (glyph, x, y) where x is left aligned, y is delta from baseline
         xmin = 0
+        ymin = math.inf
+        ymax = -math.inf
         for lineidx, glyphids in enumerate(gidlines):
-            lineglyphs = []
+            lineglyphs: list[tuple[SimpleGlyph, int, int]] = []
+
+            # Apply glyph substitutions (GSUB)
             if self.font.gsub:
                 glyphids = self.font.gsub.sub(glyphids, self.font.features)
 
             glyphs = [self.font.glyph_fromid(gid) for gid in glyphids]
-            x = 0
-            xmin = min(xmin, glyphs[0].bbox.xmin*scale)
-            for gidx, glyph in enumerate(glyphs):
-                if glyph.id not in [s.attrib['id'] for s in symbols]:
-                    symbols.append(glyph.svgsymbol())
 
-                prevglyph = glyphs[gidx-1] if gidx-1 >= 0 else None
-                nextglyph = glyphs[gidx+1] if gidx+1 < len(glyphs) else None
-                if self.font.gpos and prevglyph:
-                    # Attach marks
-                    placexy = self.font.gpos.placemark(prevglyph.index, glyph.index)
-                    if placexy:
-                        if not placexy.mkmk:  # Relative to base glyph
-                            dx = dy = 0
-                        dx += (placexy.dx - xadvance) * scale
-                        dy += -placexy.dy * scale
-                    else:
-                        dx = dy = 0
-                else:
-                    dx = dy = 0  # Don't reset, may need these for mark-to-mark
+            # Apply Glyph Positioning (GPOS)
+            if self.font.gpos:
+                xy = self.font.gpos.position(glyphs, self.font.features)
+            else:
+                xs: list[int] = list(accumulate([glyph.advance() for glyph in glyphs], initial=0))
+                xy = [(x, 0) for x in xs[:-1]]
 
-                lineglyphs.append((glyph, x+dx, dy))                
-                xadvance = glyph.advance(nextglyph)
-                x += xadvance * scale
+            lineglyphs = [(glyph, x*scale, -y*scale) for glyph, (x, y) in zip(glyphs, xy)]
 
-            if glyph.bbox.xmax > xadvance:
-                # Make linewidth a bit wider to grab right edge
-                # that extends beyond advance width
-                x += (glyph.bbox.xmax - xadvance) * scale
-                x += (-glyphs[0].bbox.xmin) * scale
-            linewidths.append(x)
+            # Accumulate multi-line widths/heights
+            if glyphs:
+                xmin = min(xmin, glyphs[0].bbox.xmin*scale)
+                last_advance = glyphs[-1].advance()
+                linewidth = (xy[-1][0] + last_advance) * scale
+                if glyphs[-1].bbox.xmax > last_advance:
+                    # Make linewidth a bit wider to grab right edge
+                    # that extends beyond advance width
+                    linewidth += (glyphs[-1].bbox.xmax - last_advance) * scale
+                    linewidth += (-glyphs[0].bbox.xmin) * scale
+            else:
+                # Blank line
+                linewidth = 0
+            linewidths.append(linewidth)
             allglyphs.append(lineglyphs)
 
+            # Accumulate SVG symbols for each glyph
+            symbols.extend([glyph.svgsymbol()
+                            for glyph in glyphs if glyph.id not in [s.attrib['id'] for s in symbols]])
+
         # Place the glyphs based on halign
         word = ET.Element('g')
         totwidth = max(linewidths)
@@ -630,8 +638,8 @@ class Text:
                 if elm is not None:
                     word.append(elm)
 
-        ymax = yvals[-1] - min(glyph[0].bbox.ymin for glyph in allglyphs[-1])*scale
-        ymin = yvals[0] - max(glyph[0].bbox.ymax for glyph in allglyphs[0])*scale
+        ymax = yvals[-1] - min(glyph[0].bbox.ymin*scale - glyph[2] for glyph in allglyphs[-1])
+        ymin = yvals[0] - max(glyph[0].bbox.ymax*scale - glyph[2] for glyph in allglyphs[0])
 
         if not config.svg2:
             symbols = []
